#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Wed Feb  9 19:13:28 2011

@ author:                  Sat Kumar Tomer 
@ author's webpage:        http://civil.iisc.ernet.in/~satkumar/
@ author's email id:       satkumartomer@gmail.com
@ author's website:        www.ambhas.com

"""
from __future__ import division
from scipy.stats import kendalltau, pearsonr, spearmanr
import numpy as np
from scipy.integrate import quad
from scipy.optimize import fmin
import sys
import statistics as st
from scipy.interpolate import interp1d

#from scipy.stats import *
#import random as R
#from pylab import*
#from scipy.interpolate import interp1d


class Copula():
    """
    This class estimate parameter of copula
    generate joint random variable for the parameters
    This class has following three copulas:
        Clayton
        Frank
        Gumbel
        
    Example:
        x = np.random.normal(size=100)
        y = np.random.normal(size=100)
        foo = Copula(x, y, 'frank')
        u,v = foo.generate(100)
    """
    

    def __init__(self, X, Y, family):
        """ initialise the class with X and Y
        Input:
            X:        one dimensional numpy array
            Y:        one dimensional numpy array
            family:   clayton or frank or gumbel
            
            Note: the size of X and Y should be same
        """
        # check dimension of input arrays
        if not ((X.ndim==1) and (Y.ndim==1)):
            raise ValueError('The dimension of array should be one.')
        
        # input array should have same zie
        if X.size is not Y.size:
            raise ValueError('The size of both array should be same.')
        
        # check if the name of copula family correct
        copula_family = ['clayton', 'frank', 'gumbel']
        if family not in copula_family:
            raise ValueError('The family should be clayton or frank or gumbel')
        
        self.X = X
        self.Y = Y
        self.family = family
        
        # estimate Kendall'rank correlation
        tau = kendalltau(self.X, self.Y)[0]
        self.tau = tau        
        
        # estimate pearson R and spearman R
        self.pr = pearsonr(self.X, self.Y)[0]
        self.sr = spearmanr(self.X, self.Y)[0]
        
        # estimate the parameter of copula
        self._get_parameter()
        
        # set U and V to none
        self.U = None
        self.V = None
        
        
    def _get_parameter(self):
        """ estimate the parameter (theta) of copula
        """        
        
        if self.family == 'clayton':
            self.theta = 2*self.tau/(1-self.tau)
            
        elif self.family == 'frank':
            self.theta = -fmin(self._frank_fun, -5, disp=False)[0]
            
        elif self.family == 'gumbel':
            self.theta = 1/(1-self.tau)
    
    def generate_uv(self, n=1000):
        """
        Generate random variables (u,v)
        
        Input:
            n:        number of random copula to be generated
        
        Output:
            U and V:  generated copula
            
        """
        # CLAYTON copula
        if self.family == 'clayton':
            U = np.random.uniform(size = n)
            W = np.random.uniform(size = n)
                
            if self.theta <= -1:
                raise ValueError('the parameter for clayton copula should be more than -1')
            elif self.theta==0:
                raise ValueError('The parameter for clayton copula should not be 0')
                
            if self.theta < sys.float_info.epsilon :
                V = W
            else:
                V = U*(W**(-self.theta/(1 + self.theta)) - 1 + U**self.theta)**(-1/self.theta)
    
        # FRANK copula
        elif self.family == 'frank':
            U = np.random.uniform(size = n)
            W = np.random.uniform(size = n)
            
            if self.theta == 0:
                raise ValueError('The parameter for frank copula should not be 0')
            
            if abs(self.theta) > np.log(sys.float_info.max):
                V = (U < 0) + np.sign(self.theta)*U
            elif abs(self.theta) > np.sqrt(sys.float_info.epsilon):
                V = -np.log((np.exp(-self.theta*U)*(1-W)/W + np.exp(-self.theta)
                            )/(1 + np.exp(-self.theta*U)*(1-W)/W))/self.theta
            else:
                V = W
        
        # GUMBEL copula
        elif self.family == 'gumbel':
            if self.theta <= 1 :
                raise ValueError('the parameter for GUMBEL copula should be greater than 1')
            if self.theta < 1 + sys.float_info.epsilon:
                U = np.random.uniform(size = n)
                V = np.random.uniform(size = n)
            else:
                u = np.random.uniform(size = n)
                w = np.random.uniform(size = n)
                w1 = np.random.uniform(size = n)
                w2 = np.random.uniform(size = n)
                
                u = (u - 0.5) * np.pi
                u2 = u + np.pi/2;
                e = -np.log(w)
                t = np.cos(u - u2/self.theta)/ e
                gamma = (np.sin(u2/self.theta)/t)**(1/self.theta)*t/np.cos(u)
                s1 = (-np.log(w1))**(1/self.theta)/gamma
                s2 = (-np.log(w2))**(1/self.theta)/gamma
                U = np.array(np.exp(-s1))
                V = np.array(np.exp(-s2))
        
        self.U = U
        self.V = V        
        return U,V
    
    def generate_xy(self, n=1000):
        """
        Generate random variables (x, y)
        
        Input:
            n:        number of random copula to be generated
        
        Output:
            X1 and Y1:  generated copula random numbers
            
        """
        # if U and V are not already generated
        if self.U is None:
            self.generate_uv(n)
            
        # estimate inverse cdf of x andy
        self._inverse_cdf()
        
        # estimate X1 and Y1        
        X1 = self._inv_cdf_x(self.U)
        Y1 = self._inv_cdf_y(self.V)
        
        return X1, Y1
        
        
    def _inverse_cdf(self):
        """
        This module will calculate the inverse of CDF 
        which will be used in getting the ensemble of X and Y from
        the ensemble of U and V
        
        The statistics module is used to estimate the CDF, which uses
        kernel methold of cdf estimation
        
        To estimate the inverse of CDF, interpolation method is used, first cdf 
        is estimated at 100 points, now interpolation function is generated 
        to relate cdf at 100 points to data
        """
        x2, x1 = st.cpdf(self.X, kernel = 'Epanechnikov', n = 100)
        self._inv_cdf_x = interp1d(x2, x1)
        
        y2, y1 = st.cpdf(self.Y, kernel = 'Epanechnikov', n = 100)
        self._inv_cdf_y = interp1d(y2, y1)
        
    
    def _integrand_debye(self,t):
         """ 
         Integrand for the first order debye function
         """
         return t/(np.exp(t)-1)
         
    def _debye(self, alpha):
        """
        First order Debye function
        """
        return quad(self._integrand_debye, sys.float_info.epsilon, alpha)[0]/alpha
    
    def _frank_fun(self, alpha):
        """
        optimization of this function will give the parameter for the frank copula
        """
        diff = (1-self.tau)/4.0  - (self._debye(-alpha)-1)/alpha
        return diff**2
        
        











#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Thu Jan 20 15:36:37 2011
@ author:                  Sat Kumar Tomer 
@ author's webpage:        http://civil.iisc.ernet.in/~satkumar/
@ author's email id:       satkumartomer@gmail.com
@ author's website:        www.ambhas.com

A libray with Python functions for calculations of 
micrometeorological parameters and some miscellaneous
utilities.

functions:
    pc_bias : percentage bias
    apb :     absolute percent bias
    rmse :    root mean square error
    mae :     mean absolute error
    bias :    bias
    NS :      Nash-Sutcliffe Coefficient
    
"""

# import required modules
import numpy as np

def filter_nan(s,o):
    data = np.array([s,o])
    data = np.transpose(data)
    data = data[~np.isnan(data).any(1)]
    return data[:,0],data[:,1]

def pc_bias(s,o):
    s,o = filter_nan(s,o)
    return 100.0*sum(s-o)/sum(o)

def apb(s,o):
    s,o = filter_nan(s,o)
    return 100.0*sum(abs(s-o))/sum(o)

def rmse(s,o):
    s,o = filter_nan(s,o)
    return np.sqrt(np.mean((s-o)**2))

def mae(s,o):
    s,o = filter_nan(s,o)
    return np.mean(abs(s-o))

def bias(s,o):
    s,o = filter_nan(s,o)
    return np.mean(s-o)

def NS(s,o):
    s,o = filter_nan(s,o)
    return 1 - sum((s-o)**2)/sum((o-np.mean(o))**2)

def L(s,o, N=5):
    s,o = filter_nan(s,o)
    return np.exp(-N*sum((s-o)**2)/sum((o-np.mean(o))**2))

def correlation(s,o):
    s,o = filter_nan(s,o)
    return np.corrcoef(o, s)[0,1]




#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Mon Feb  7 11:53:29 2011
@author: sat kumar tomer (http://civil.iisc.ernet.in/~satkumar/)

"""

import numpy as np

class sun():
    """
    This class estimate extral-terrestial solar radiation,
    sunset time, sunrise time, daylight hours
    
    Based on the article:
        Empirical Model for Estimating Global Solar Radiation
        on Horizontal Surfaces for Selected Cities in 
        the Six Geopolitical Zones in Nigeria
    By:
        M.S. Okundamiya and A.N. Nzeako

        
    Example:
        lat_deg = 11
        lon_deg = 76
        doy = 180
        foo = sun(doy, lat_deg, lon_deg)
        hourly_etr = foo.hourly_ETR(11.0)
        daily_etr = foo.daily_ETR()
        risetime, settime = foo.set_rise()
    """
    

    def __init__(self, doy, lat_deg, lon_deg, ze=5.5):
        """ initialise the class
        Input:
            doy:            day of year
            lat_deg:        latitude in degree ( south is negative)
            lon_deg:        longitude in degree (west is negative)
            ze :            time zone in hour
        """
        self.doy = doy
        self._lat_rad = lat_deg*np.pi/180
        
        self.lon_deg = lon_deg
        self.ze = ze

        
        
        self._Isc = 1367 # solar constant (W/m^2)

        B_deg = (doy-1)*360/365
        B_rad = B_deg*np.pi/180
        
        E = 3.82*(0.000075 + 0.001868*np.cos(B_rad) - 0.032077*np.sin(B_rad) \
                    - 0.0141615*np.cos(2*B_rad) -0.04089*np.sin(2*B_rad) )
        
        self._E = E        
        
        # declination
        delta = 0.4093*np.sin(2*np.pi*(284+doy)/365)
        self._delta = delta
    
        # relative distance of the earth from sun
        dr = 1 + 0.0033*np.cos(2*np.pi*doy/365)
        self.dr = dr
        
    def hourly_ETR(self,tc):
        """
        Input:
            tc:     local time in hours
        """
        ze = self.ze
        lat_rad = self._lat_rad
        E = self._E 
        delta = self._delta
        dr = self.dr
        
        ts = tc + E + self.lon_deg/15 - ze
        # hour angle
        w = ((ts-12)*15)/180.0*np.pi
        # hourly ETR
        I0 = self._Isc*dr*(np.sin(lat_rad)*np.sin(delta) + 
                np.cos(lat_rad)*np.cos(delta)*np.cos(w))
        
        return I0
        
    def daily_ETR(self):
        """ 
        Returns the daily Extra Terrestial Radiation (ETR)
        Input:
            
        Output:
            
        """
        lat_rad = self._lat_rad
        delta = self._delta
        
        # sunset hour angle
        ws = np.arccos(-np.tan(lat_rad)*np.tan(delta))
        # maximum sunshine hour lenght
        self.N = ws*24/np.pi

        H0 = (1/np.pi)*self._Isc*self.dr*(ws*np.sin(delta)*np.sin(lat_rad) +
                np.cos(delta)*np.cos(lat_rad))
                
        return H0
    
    def set_rise(self):
        """
        Returns the sun set and rise time
        """
        lon_deg = self.lon_deg
        lat_rad = self._lat_rad
        delta = self._delta
        E = self._E
        ze = self.ze
        
        w1 = np.arccos(- np.tan(delta)*np.tan(lat_rad))
        ts1 = 12 - w1*180/np.pi/15 
        ts2 = 12 + w1*180/np.pi/15 
        
        tc1 = ts1 - E - lon_deg/15 + ze
        tc2 = ts2 - E - lon_deg/15 + ze
        
        return tc1, tc2



# -*- coding: utf-8 -*-
"""
Created on Wed Nov 16 14:25:50 2011

@author: Sat Kumar Tomer
@website: www.ambhas.com
@email: satkumartomer@gmail.com
"""
from __future__ import division
import numpy as np
from ambhas.err import L

class GW_1D():
    """
    This class perform the groundwater modelling
    """
    
    def __init__(self, R, Dnet=0):
        """
        R: rainfall
        Dnet: net groundwater draft
        """
        self.R = R
        self.Dnet = 0
        
    def set_parameters(self, F, G, r, hmin=0):
        """
        F: model parameter
        G: model parameter
        r: recharge factor
        hmin: groundwater level at which based flow ceases
        
        sy: specific yield
        lam: decay constant
        """
        self.F = F
        self.G = G
        self.r = r
        self.hmin = hmin
        
        self.lam = (1-F)**2/G
        self.sy = (1-F)/G
            
    def run_model(self, hini, t):
        """
        hini: initial groundwater level
        t: time
        """
        u = self.r*self.R-self.Dnet # net input
        
        h = np.empty(t+1) # create empty array
        h[0] = hini  - self.hmin       # set the initial condition
                
        for k in range(t):
            h[k+1] = self.F*h[k] + self.G*u[k]
            
            
        self.h = h + self.hmin
    
    def ens(self, F_lim, G_lim, r_lim, hmin_lim, ens, hini, h_obs, t):
        """
        generate ensemble based on ensemble of parameters
        Input:
            F: min and max of F
            G: min and max of G
            r: min and max of r
            hmin: min and max of hmin
            ens: no. of ensembles
            hini: initial gw level
            t: final time
        """
        F_ens = F_lim[0] + (F_lim[1]-F_lim[0]) * np.random.rand(ens)
        G_ens = G_lim[0] + (G_lim[1]-G_lim[0]) * np.random.rand(ens)
        r_ens = r_lim[0] + (r_lim[1]-r_lim[0]) * np.random.rand(ens)
        hmin_ens = hmin_lim[0] + (hmin_lim[1]-hmin_lim[0]) * np.random.rand(ens)
        
        self.NS = np.empty(ens)
        for i in range(ens):
            self.set_parameters(F_ens[i], G_ens[i], r_ens[i], hmin_ens[i])
            self.run_model(hini, t)
            self.NS[i] = NS(self.h, h_obs[:t+1])
   
#if __name__ == "__main__":
#    # forcing
#    R = np.random.rand(100)
#    foo = GW_1D(R)
#    
#    # parameter
#    F = 0.8
#    G = 10
#    r = 0.1
#    #foo.set_parameters(F, G, r)
#    
#    # initial condition
#    h_ini = 10
#    
#    # run the model    
#    #foo.run_model(h_ini, 100)
#    
#    # get the simulated data
#    #h_sim = foo.h
#    
#    # ensemble
#    h_obs = np.random.rand(101)
#    foo.ens([0.5, 1.0], [1, 20], [0, 0.2], [50,70], 10, 20, h_obs, 10)
#    print(foo.NS)
    
    
    


#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 25 15:49:45 2011

@ author:                  Sat Kumar Tomer 
@ author's webpage:        http://civil.iisc.ernet.in/~satkumar/
@ author's email id:       satkumartomer@gmail.com
@ author's website:        www.ambhas.com
"""

# import required library
import xlrd
import numpy as np

class time_data:
    def __init__(self, book_name):
        book = xlrd.open_workbook(book_name)
        sheet_daily = book.sheet_by_name('daily')
        sheet_monthly = book.sheet_by_name('monthly')
        
        # parse the daily data
        labels_daily = []
        for i in xrange(sheet_daily.ncols):
            labels_daily.append(str(sheet_daily.cell_value(0,i)))
        self.labels_daily = labels_daily
        
        data_daily = np.zeros((sheet_daily.nrows-1,sheet_daily.ncols))
        for i in xrange(sheet_daily.nrows-1):
            for j in xrange(sheet_daily.ncols):
                try:
                    data_daily[i,j] = sheet_daily.cell_value(i+1,j)
                except:
                    data_daily[i,j] = np.nan
        self.data_daily = data_daily
        
        # parse the monthly data
        labels_monthly = []
        for i in xrange(sheet_monthly.ncols):
            labels_monthly.append(str(sheet_monthly.cell_value(0,i)))
        self.labels_monthly = labels_monthly
        
        data_monthly = np.zeros((sheet_monthly.nrows-1,sheet_monthly.ncols))
        for i in xrange(sheet_monthly.nrows-1):
            for j in xrange(sheet_monthly.ncols):
                try:
                    data_monthly[i,j] = sheet_monthly.cell_value(i+1,j)
                except:
                    data_monthly[i,j] = np.nan
        self.data_monthly = data_monthly
        
    def get(self, var_name):
        if var_name in self.labels_daily:
            var_data = self.data_daily[:,self.labels_daily.index(var_name)]
        elif var_name in self.labels_monthly:
            var_data = self.data_monthly[:,self.labels_monthly.index(var_name)]
        else:
            raise ValueError, var_name + ' not found in the worksheet'
        return var_data
    
    def get_monthly(self, var_name, var_operation = 'mean'):
        if var_name in self.labels_daily:
            year = self.data_daily[:,self.labels_daily.index('year')]
            month = self.data_daily[:,self.labels_daily.index('month')]
            var_data = self.data_daily[:,self.labels_daily.index(var_name)]
            
            min_year = year[0]
            max_year = year[-1]
            
            var_monthly = np.zeros(((max_year - min_year +1)*12,1))
            i = 0
            for ind_year in xrange(min_year,max_year+1):
                for ind_month in xrange(1,13):
                    #var_monthly[i] = sum(var_data[(year == ind_year) & (month == ind_month)])
                    exec('var_monthly[i] = '+var_operation+'(var_data[(year == ind_year) & (month == ind_month)])')
                    i += 1          
                     
            return var_monthly
    
    def get_yearly(self, var_name, var_operation = 'mean'):
        if var_name in self.labels_daily:
            year = self.data_daily[:,self.labels_daily.index('year')]
            var_data = self.data_daily[:,self.labels_daily.index(var_name)]
            
            min_year = year[0]
            max_year = year[-1]
            
            var_yearly = np.zeros((max_year - min_year +1,1))
            i = 0
            for ind_year in xrange(min_year,max_year+1):
                exec('var_yearly[i] = '+var_operation+'(var_data[year == ind_year])')
                i += 1     
        
        elif var_name in self.labels_monthly:
            year = self.data_monthly[:,self.labels_monthly.index('year')]
            var_data = self.data_monthly[:,self.labels_monthly.index(var_name)]
            
            min_year = year[0]
            max_year = year[-1]
            
            var_yearly = np.zeros((max_year - min_year +1,1))
            i = 0
            for ind_year in xrange(min_year,max_year+1):
                #var_yearly[i] = sum(var_data[year == ind_year])
                exec('var_yearly[i] = '+var_operation+'(var_data[year == ind_year])')
                i += 1        
                     
        return var_yearly
    
    def nor(self, season = 'yearly', threshold = 10.0):
        year = self.data_daily[:,self.labels_daily.index('year')]
        month = self.data_daily[:,self.labels_daily.index('month')]
        rain_daily = self.get('rainfall')
        
        min_year = int(year[0])
        max_year = int(year[-1])
        
        if season == 'monthly':
            # monthly distribution of rainy days
            nor_monthly = np.zeros(((max_year - min_year +1)*12,1))
            i = 0
            for ind_year in xrange(min_year,max_year+1):
                    for ind_month in xrange(1,13):
                        nor_monthly[i] = sum(rain_daily[(year == ind_year) & (month == ind_month)]>threshold)
                        i += 1  
            return nor_monthly
        elif season == 'yearly':
            # yearly distribution of rainy days
            nor_yearly = np.zeros((max_year - min_year +1,1))
            i = 0
            for ind_year in xrange(min_year,max_year+1):
                nor_yearly[i] = sum(rain_daily[year == ind_year]>threshold)
                i += 1  
            return nor_yearly


# -*- coding: utf-8 -*-
"""
Created on Fri Nov 11 16:49:46 2011

@author: Sat Kumar Tomer
@website: www.ambhas.com
@email: satkumartomer@gmail.com
"""

import xlrd, xlwt
import numpy as np

class xlsread():
    """
    A class to read data from xls file
    based on the 'xlrd'
    """
    
    def __init__(self, fname):
        self.fname = fname
                
    def get_cells(self, cell_range, sheet):
        """
        cell_range: a single cell i.e. 'a2'
                    range of cells i.e. 'a2:f5'
        sheet:  name of the sheet, must be string
        """
        
        book = xlrd.open_workbook(self.fname)
        sheet = book.sheet_by_name(sheet)
        
        if ':' not in cell_range:
            foo1 = cell_range
            row,col = self.__cell2ind__(foo1)
            data = sheet.cell_value(row,col)
        else:
            foo1, foo2 = cell_range.split(':')
            row1,col1 = self.__cell2ind__(foo1)
            row2,col2 = self.__cell2ind__(foo2)
            
            if row2<row1:
                raise Exception('row_start should be <= row_end')
            if col2<col1:
                raise Exception('col_start should be <= col_end')
            
            data = []
            for i in range(row1,row2+1):
                data_row = []
                for j in range(col1,col2+1):
                    data_row.append(sheet.cell_value(i,j))
                    
                data.append(data_row)
            
        return np.array(data)

    def __cell2ind__(self,foo):
        """
        given the cell number i.e. (AA100)
        returns the row and column of cell in indices format i.e. 0,10
        """
        
        # if the column is less than Z
        foo_str = foo[0].lower()
        col = ord(foo_str)-ord('a')
        
        # if the column is more than Z, i.e. AA
        try:
            row = int(foo[1:])-1
        except:
            row = int(foo[2:])-1
            foo_str = foo[1].lower()
            col = (col+1)*26+ ord(foo_str)-ord('a')
        
        return row, col


class xlswrite():
    
    def __init__(self, data, cell_start, sheet):
        self.data = data
        self.cell_start = cell_start
        
        # initialize the xlrt     
        book = xlwt.Workbook()
        sheet = book.add_sheet(sheet)
        
        # convert into row and col        
        row, col = self.__cell2ind__(cell_start)
        
        if isinstance(data, str)  or isinstance(data, float) or isinstance(data,int):
            sheet.write(row,col,data)
                
        if data.ndim == 1:
            for i in range(data.shape[0]):
                sheet.write(row+i,col, data[i])
                
        else:
            for i in range(data.shape[0]):
                for j in range(data.shape[1]):
                    sheet.write(row+i, col+j, data[i,j])
        
        self.book = book
    
    def save(self, fname):
        self.book.save(fname)
    
    
    def __cell2ind__(self, foo):
        """
        given the cell number i.e. (AA100)
        returns the row and column of cell in indices format i.e. 0,10
        """
        
        # if the column is less than Z
        foo_str = foo[0].lower()
        col = ord(foo_str)-ord('a')
        
        # if the column is more than Z, i.e. AA
        try:
            row = int(foo[1:])-1
        except:
            row = int(foo[2:])-1
            foo_str = foo[1].lower()
            col = (col+1)*26+ ord(foo_str)-ord('a')
        
        return row,col
        
if __name__ == "__main__":
    
    # read the data
    fname = '/home/tomer/rain_projection/raw_data/a2_0.5.xls'
        
    foo = xlsread(fname)
    var = foo.get_cells('a3:a5', 'Sheet1')
    
    book = xlrd.open_workbook(fname)
    sheet = book.sheet_by_name('Sheet1')
    sheet.cell_value(0,0)    
    
    print var
    
    # write the data
    var = np.array([[5,10,12],[2,5,6]])
    foo1 = xlswrite(var, 'f10', 'Sheet1')
    fname = '/home/tomer/data.xls'
    foo1.save(fname)
    
    




#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 15 19:54:08 2010

@author: sat kumar tomer (http://civil.iisc.ernet.in/~satkumar/)

Functions:
    utm2deg: Calculate utm co-ordinates from Lat, Lon
    deg2utm : Calculate Lat, Lon from UTM
    kabini: 
    berambadi: 
"""

# load needed python modules 
from pyproj import Proj



def Geo2Pixel(Xgeo,Ygeo,GT):
    a1 = GT[1]
    b1 = GT[2]
    c1 = Xgeo - GT[0]
    a2 = GT[4]
    b2 = GT[5]
    c2 = Ygeo - GT[3]
    Xpixel = (b2*c1-b1*c2)/(a1*b2-a2*b1) 
    Yline = (a2*c1-a1*c2)/(a2*b1-a1*b2) 
    return Xpixel, Yline

def Pixel2Geo(Xpixel,Yline,GT):
    Xgeo = GT[0] + Xpixel*GT[1] + Yline*GT[2]
    Ygeo = GT[3] + Xpixel*GT[4] + Yline*GT[5]
    return Xgeo,Ygeo

def SetProjectionBerambadi():
	return 'PROJCS["WGS 84 / UTM zone 43N",GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84",6378137,298.257223563,AUTHORITY["EPSG","7030"]],AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0],UNIT["degree",0.0174532925199433],AUTHORITY["EPSG","4326"]],PROJECTION["Transverse_Mercator"],PARAMETER["latitude_of_origin",0],PARAMETER["central_meridian",75],PARAMETER["scale_factor",0.9996],PARAMETER["false_easting",500000],PARAMETER["false_northing",0],UNIT["metre",1,AUTHORITY["EPSG","9001"]],AUTHORITY["EPSG","32643"]]'

def utm2deg(x,y,utmzone=43):
    p = Proj(proj='utm',zone=43,ellps='WGS84')
    Lon,Lat = p(x, y, inverse=True)
    return Lon,Lat
    
def deg2utm(Lon,Lat,utmzone=43):
    p = Proj(proj='utm',zone=utmzone,ellps='WGS84')
    x,y = p(Lon, Lat)
    return x,y
    
def kabini(Ifile,Ofile):
    # define the file names file
    Temp1 = '/home/tomer/MODISdata/temp/temp1.tif'
    Temp2 = '/home/tomer/MODISdata/temp/temp2.tif'
            
    # convert from geographical co-ordinates to UTM zone 43
    cmd= 'gdalwarp -r bilinear -t_srs  \'+proj=utm +zone=43 +datum=WGS84\' ' + Ifile + ' ' +Temp1
    try:
        returncode = call(cmd, shell=True)
        if returncode:
            print 'Failure with returncode', returncode; sys.exit(1)
    except OSError, message:
        print 'Execution failed!\n', message; sys.exit(1)

    # cut the area around Kabini Basin (output image will be of size 129 X 102)
    kab='582000 1372000 711000 1270000'
    cmd = 'gdal_translate -a_ullr ' + kab + ' -projwin ' + kab + ' ' + Temp1+ ' ' + Temp2
    try:
        returncode = call(cmd, shell=True)
        if returncode:
            print 'Failure with returncode', returncode; sys.exit(1)
    except OSError, message:
        print 'Execution failed!\n', message; sys.exit(1)
        
    # changing the resolution
    cmd = 'gdalwarp -r bilinear -tr 1000 1000 ' + Temp2 + ' ' +Ofile
    try:
        returncode = call(cmd, shell=True)
        if returncode:
            print 'Failure with returncode', returncode; sys.exit(1)
    except OSError, message:
        print 'Execution failed!\n', message; sys.exit(1)
        
    # remove the temporary file
    try:
        os.remove(Temp)
        os.remove(Temp2)
    except:
        print 'temp file not created'

def berambadi(Ifile,Ofile):
    # define the file names file
    Temp1 = '/home/tomer/MODISdata/temp/temp1.tif'
    Temp2 = '/home/tomer/MODISdata/temp/temp2.tif'
            
    # convert from geographical co-ordinates to UTM zone 43
    cmd= 'gdalwarp -r bilinear -t_srs  \'+proj=utm +zone=43 +datum=WGS84\' ' + Ifile + ' ' +Temp1
    try:
        returncode = call(cmd, shell=True)
        if returncode:
            print 'Failure with returncode', returncode; sys.exit(1)
    except OSError, message:
        print 'Execution failed!\n', message; sys.exit(1)

    # cut the area around Kabini Basin (output image will be of size 129 X 102)
    bmc='664000 1309000 685000 1294000'
    cmd = 'gdal_translate -a_ullr ' + bmd + ' -projwin ' + bmd + ' ' + Temp1+ ' ' + Temp2
    try:
        returncode = call(cmd, shell=True)
        if returncode:
            print 'Failure with returncode', returncode; sys.exit(1)
    except OSError, message:
        print 'Execution failed!\n', message; sys.exit(1)
        
    # changing the resolution
    cmd = 'gdalwarp -r bilinear -tr 1000 1000 ' + Temp2 + ' ' +Ofile
    try:
        returncode = call(cmd, shell=True)
        if returncode:
            print 'Failure with returncode', returncode; sys.exit(1)
    except OSError, message:
        print 'Execution failed!\n', message; sys.exit(1)
        
    # remove the temporary file
    try:
        os.remove(Temp1)
        os.remove(Temp2)
    except:
        print 'temp file not created'




# -*- coding: utf-8 -*-
"""
Created on Thu Jun  9 17:55:54 2011

@author: Sat Kumar Tomer
@website: www.ambhas.com
@email: satkumartomer@gmail.com
"""

# import required modules
import numpy as np
import matplotlib.pylab as plt


class OK:
    """
    This performs the ordinary kriging
    Input:
        x: x vector of location
        Y: y vector of location
        z: data vector at location (x,y)
    
    Output:
        None
        
    Methods:
        variogram: estimate the variogram
        
    """
    def __init__(self,x,y,z):
        self.x = x
        self.y = y
        self.z = z
    
    def variogram(self, var_type='averaged', min_lag=None):
        x = self.x
        y = self.y
        z = self.z
        # make the meshgrid
        X1,X2 = np.meshgrid(x,x) 
        Y1,Y2 = np.meshgrid(y,y)
        Z1,Z2 = np.meshgrid(z,z)
        
        D = np.sqrt((X1 - X2)**2 + (Y1 - Y2)**2)
        self.D = D
        G = 0.5*(Z1 - Z2)**2
        indx = range(len(z))
        C,R = np.meshgrid(indx,indx)
        I = R > C

        D2 = D*(np.diag(x*np.nan)+1)
        lag = 10*np.mean(np.nanmin(D2))
	if min_lag is not None:
	    lag = min_lag

        hmd = D.max()/2
        max_lags = np.floor(hmd/lag)
        LAGS = np.ceil(D/lag)
        
        DE = np.empty(max_lags)
        PN = np.empty(max_lags)
        GE = np.empty(max_lags)
        for i in range(int(max_lags)):
            SEL = (LAGS == i+1)
            DE[i] = D[SEL].mean()
            PN[i] = np.sum(SEL == 1)/2
            GE[i] = G[SEL].mean()
            
        if var_type == 'scattered':
            return D[I],G[I]      
        elif var_type == 'averaged':
            return DE,GE
        else:
            raise ValueError('var_type should be either averaged or scatter')
        
    def vario_model(self, lags, model_par, model_type='linear'):
        """
        Input:
            model_type : the type of variogram model 
                             spherical
                             linear
                             exponential
        """
        
        if model_type == 'spherical':
            n = model_par['nugget']
            r = model_par['range']
            s = model_par['sill']
            l = lags
            G = n + (s*(1.5*l/r - 0.5*(l/r)**3)*(l<=r) + s*(l>r))
        
        elif model_type == 'linear':
            n = model_par['nugget']
            s = model_par['slope']
            l = lags
            G = n + s*l
        
        elif model_type == 'exponential':
            n = model_par['nugget']
            r = model_par['range']
            s = model_par['sill']
            l = lags
            G = n + s*(1 - np.exp(-3*l/r))
        
        else:
            raise ValueError('model_type should be spherical or linear or exponential')
            
        return G
            
    def krige(self, Xg, Yg, model_par, model_type):
        
        # set up the Gmod matrix 
        n = len(self.x)
        Gmod = np.empty((n+1,n+1))
        Gmod[:n, :n] = self.vario_model(self.D, model_par, model_type)
                
        Gmod[:,n] = 1
        Gmod[n,:] = 1
        Gmod[n,n] = 0

        Gmod = np.matrix(Gmod)      
        
        # inverse of Gmod
        Ginv = Gmod.I

        Xg = Xg.flatten()
        Yg = Yg.flatten()        
        Zg = np.empty(Xg.shape)
        s2_k = np.empty(Xg.shape)
        
        for k in range(len(Xg)):
            
            DOR = ((self.x - Xg[k])**2 + (self.y - Yg[k])**2)**0.5
            GR = np.empty((n+1,1))
            GR[:n,0] = self.vario_model(DOR, model_par, model_type)
            GR[n,0] = 1
            E = np.array(Ginv * GR )
            Zg[k] = np.sum(E[:n,0]*self.z)
            s2_k[k] = np.sum(E[:n,0]*GR[:n,0])+ E[n, 0]
        
        self.Zg = Zg
        self.s2_k = s2_k
            
if __name__ == "__main__":          
    # generate some sythetic data
    x = np.random.normal(size=100)
    y = np.random.normal(size=100)
    z = 0.0*np.random.normal(size=100)+x+y
    
    foo = OK(x,y,z)
    ax,ay = foo.variogram()
    
    #plt.plot(ax,ay,'ro')
    
    lags = np.linspace(0,5)
    model_par = {}
    model_par['nugget'] = 0
    model_par['range'] = 1
    model_par['sill'] = 2.0
    
    G = foo.vario_model(lags, model_par, model_type = 'exponential')
    #plt.plot(lags, G, 'k')
    #plt.show()
    
    Rx = np.linspace(-1,1)
    Ry = np.linspace(0,1)
    foo.krige(Rx, Ry, model_par, 'exponential')
    
    plt.plot(x,z,'ro')
    plt.plot(Rx,foo.Zg)
    plt.show()



# -*- coding: utf-8 -*-
"""
Created on Tue May 24 18:07:28 2011

@author: Sat Kumar Tomer
@website: www.ambhas.com
@email: satkumartomer@gmail.com
"""

# import required libraries
from __future__ import division
import numpy as np
from errlib import rmse
from scipy.optimize import fmin
from scipy.stats import poisson

class RainDisagg:
    
    
    def __init__(self,rf):
        self.rf = rf
            
        # calculate the length of rainfall series
        len_rf = len(rf)
                  
        # take only that much length of rainfall series which is multiplication of 32 
        rf_1 = rf[0:-np.mod(len_rf,32)]
        
        # summing rainfall for 2 days
        rf_2 = np.sum(rf_1.reshape(-1,2), axis=1)
        
        # summing rainfall for 4 days
        rf_4 = np.sum(rf_2.reshape(-1,2), axis=1)
        
        # summing rainfall for 8 days
        rf_8 = np.sum(rf_4.reshape(-1,2), axis=1)
        
        # summing rainfall for 16 days
        rf_16 = np.sum(rf_8.reshape(-1,2), axis=1)
        
        # summing rainfall for 32 days
        rf_32 = np.sum(rf_16.reshape(-1,2), axis=1)
        
        #generate moments (q varies from 0 to 5)
        # row => time scale
        # column => moments
        M1 = np.zeros((6,11))
        for i in range(11):
            M1[0,i] = np.mean(rf_1**(i/2))
            M1[1,i] = np.mean(rf_2**(i/2))
            M1[2,i] = np.mean(rf_4**(i/2))
            M1[3,i] = np.mean(rf_8**(i/2))
            M1[4,i] = np.mean(rf_16**(i/2))
            M1[5,i] = np.mean(rf_32**(i/2))
        
        self.logM = np.log(M1)
        
        # l is inverse of time scale
        l = [32, 16, 8, 4, 2, 1]
        self.log_lambda = np.log(l)
        
        
        # calculate the tau from the slope of log(M) vs log(l)
        tau_obs = np.zeros(10,)
        for i in range(10):
            tau_obs[i] = -np.polyfit(np.log(l), np.log(M1[:,i+1]),1)[0]
        self.tau_obs = tau_obs
        
        # fit the log-poisson distribution
        self.lp = fmin(self.fun_poisson,np.array([0.4, 0.2]))
        
        # calculate the parameter of log poisson distribution form the parameters
        # of tau function
        self.A = np.exp(self.lp[0]*(1-self.lp[1]))

    #define the log-poisson function
    def fun_poisson(self,par):
        q = np.arange(0.5,5.5,0.5)
        c = abs(par[0])
        beta = abs(par[1])
        b = 2
        tau_pred = q-c*(q*(1-beta)+beta**q-1)/(np.log(b))
        f = rmse(tau_pred,self.tau_obs)
        return f
    
    def disaggregate(self,rf):
        len_rf = len(rf)        
        # generating rainfall from t h to t/2 h
        rf_pre = np.zeros((2,len_rf*2))
        for j in range(2):
            for i in xrange(0,len_rf*2,2):
                W = self.A*(self.lp[1])**poisson.rvs(1, size=2)
                rf_pre[j,i] = rf[int(i/2)]*W[0]/(W[0]+W[1])
                rf_pre[j,i+1] = rf[int(i/2)]*W[1]/(W[0]+W[1])
        
        rf_pre = np.mean(rf_pre, axis=0)

        # rounding up the simulated rainfall to the least count of raingauge 
        for i in xrange(0,len_rf*2,2):
            if np.mod(rf_pre[i],0.5) !=0:
                TB = np.mod(rf_pre[i],0.5)
            else:
                TB = 0
            
            rf_pre[i] -= TB
            rf_pre[i+1] += TB
        
        return rf_pre

